#include <Joystick.h>
/*
  Controller

  Reads the specified input ports regularly and outputs the values
*/

//Name the controller masks
#define coin 0b00000100   //D2

#define p1Up 0b00000010   //D1
#define p1Left 0b00000001 //D0
#define p1Down 0b00010000 //D4
#define p1Right 0b01000000 //C6
#define p1Start 0b10000000 //D7
#define p1Blue 0b01000000  //E6
#define p1Yellow 0b00010000 //B4
#define p1Black 0b00100000 //B5
#define p1Red 0b01000000 //B6

#define p2Up 0b10000000 //B7
#define p2Left 0b01000000 //D6
#define p2Down 0b10000000 //C7
#define p2Right 0b10000000 //F7
#define p2Start 0b01000000 //F6
#define p2Blue 0b00100000 //F5
#define p2Yellow 0b00010000 //F4
#define p2Black 0b00000010 //F1
#define p2Red 0b00000001 //F0

#define NUMBER_INPUTS 19
#define JOYSTICK_COUNT 2

#define MASK_B 0b11110000
#define MASK_C 0b11000000
#define MASK_D 0b11010111
#define MASK_E 0b01000000
#define MASK_F 0b11110011

const char inputsArray[NUMBER_INPUTS] = {
  0,  //coin
  2,  //p1Up
  3,  //p1Left
  4,  //p1Down
  5,  //p1Right
  6,  //p1Start
  7,  //p1Blue
  8,  //p1Yellow
  9,  //p1Black
  10, //p1Red
  11, //p2Up
  12, //p2Left
  13, //p2Down
  18, //p2Right
  19, //p2Start
  20, //p2Blue
  21, //p2Yellow
  22, //p2Black
  23  //p2Red
};

Joystick_ Joystick[JOYSTICK_COUNT] = {
Joystick_(0x03,JOYSTICK_TYPE_GAMEPAD,
  6, 0,                  // Button Count, Hat Switch Count
  true, true, false,     // X and Y, but no Z Axis
  false, false, false,   // No Rx, Ry, or Rz
  false, false,          // No rudder or throttle
  false, false, false),  // No accelerator, brake, or steering

Joystick_(0x04,JOYSTICK_TYPE_GAMEPAD,
  5, 0,                  // Button Count, Hat Switch Count
  true, true, false,     // X and Y, but no Z Axis
  false, false, false,   // No Rx, Ry, or Rz
  false, false,          // No rudder or throttle
  false, false, false)   // No accelerator, brake, or steering
};

void configureInputs(){
  //Configures all inputs and sets pull-up resistors
  for(int i = 0; i < NUMBER_INPUTS; i++){
    pinMode(inputsArray[i], INPUT_PULLUP);
  }
}

void setup() {
  configureInputs();
  
  for (int index = 0; index < JOYSTICK_COUNT; index++)
  {
    // Initialize Joystick Library
    Joystick[index].begin();
    Joystick[index].setXAxisRange(-1, 1);
    Joystick[index].setYAxisRange(-1, 1);
  }
}
// Last state of the buttons
char lastB = 0;
char lastC = 0;
char lastD = 0;
char lastE = 0;
char lastF = 0;

// New state of the buttons
char newB = 0;
char newC = 0;
char newD = 0;
char newE = 0;
char newF = 0;

// Which of the buttons got a new state
char xorB = 0;
char xorC = 0;
char xorD = 0;
char xorE = 0;
char xorF = 0;

void loop() {
  //get new inputs and negate them (1 if pressed, 0 if not)
  newB = MASK_B & ~PINB;
  newC = MASK_C & ~PINC;
  newD = MASK_D & ~PIND;
  newE = MASK_E & ~PINE;
  newF = MASK_F & ~PINF;
  
  //check for what changed (0 if no, 1 if yes)
  xorB = newB ^ lastB;
  xorC = newC ^ lastC;
  xorD = newD ^ lastD;
  xorE = newE ^ lastE;
  xorF = newF ^ lastF;

  // The outer ifs aren't totally necessary, but they could 
  // shave off some of the latency in the average case.
  // They could also add some though, so maybe we should check that
  // (Also, this doesn't do debouncing yet, which I don't like)
  if(xorB){
    if(xorB & p1Yellow){
      Joystick[0].setButton(3, newB & p1Yellow);
    }
    if(xorB & p1Black){
      Joystick[0].setButton(0, newB & p1Black);
    }
    if(xorB & p1Red){
      Joystick[0].setButton(1, newB & p1Red);
    }
  }

  if(xorD){
    if(xorD & coin){
      Joystick[0].setButton(5, newD & coin);
    }
    if(xorD & p1Start){
      Joystick[0].setButton(4, newD & p1Start);
    }
  }

  if(xorE & p1Blue){
    Joystick[0].setButton(2, newE & p1Blue);
  }

  if(xorF){
    if(xorF & p2Red){
      Joystick[1].setButton(1, newF & p2Red);
    }
    if(xorF & p2Black){
      Joystick[1].setButton(0, newF & p2Black);
    }
    if(xorF & p2Yellow){
      Joystick[1].setButton(3, newF & p2Yellow);
    }
    if(xorF & p2Blue){
      Joystick[1].setButton(2, newF & p2Blue);
    }
    if(xorF & p2Start){
      Joystick[1].setButton(4, newF & p2Start);
    }
  }

  //Player 1 X
  if((xorC & p1Right) | (xorD & p1Left)){
    if (newC & p1Right) {
      Joystick[0].setXAxis(1);
    } else if (newD & p1Left) {
      Joystick[0].setXAxis(-1);
    } else {
      Joystick[0].setXAxis(0);
    }
  }

  //Player 1 Y
  if(xorD & (p1Down | p1Up)){
    if(newD & p1Down) {
      Joystick[0].setYAxis(1);
    } else if (newD & p1Up) {
      Joystick[0].setYAxis(-1);
    } else {
      Joystick[0].setYAxis(0);
    }
  }

  // Player 2 X
  if((xorF & p2Right) | (xorD & p2Left)){
    if (newF & p2Right) {
      Joystick[1].setXAxis(1);
    } else if (newD & p2Left) {
      Joystick[1].setXAxis(-1);
    } else {
      Joystick[1].setXAxis(0);
    }
  }

  // Player 2 Y 
  if((xorB & p2Up) | (xorC & p2Down)){
    if (newC & p2Down) {
      Joystick[1].setYAxis(1);
    } else if (newB & p2Up) {
      Joystick[1].setYAxis(-1);
    } else {
      Joystick[1].setYAxis(0);
    }
  }

  lastB = newB;
  lastC = newC;
  lastD = newD;
  lastE = newE;
  lastF = newF;
  
}
