#include "Adafruit_Arcada.h"
#include "Adafruit_GFX.h"


Adafruit_Arcada arcada;
#define TIMEOUT 1000000

//TODO: Write a Window Manager, as strange as it sounds
//A "color Scheme"!
#define COLOR_BG ARCADA_BLACK
#define COLOR_TEXT ARCADA_BLACK
#define COLOR_ACCENT ARCADA_GREEN

//Screen max values
#define SC_XMAX 159
#define SC_YMAX 127

uint16_t light_threshold = 400; //Threshold of the photoresistor
uint16_t pulsewidth_ms = 1;     //width of the generated input pulse
uint16_t downtime_ms = 150;     //length of the pause between two measurements

//selected menu entry
uint8_t selected = 1;

bool setupMode = false;
bool measurementRunning = true;

//active: generate button press pulse, passive: wait for button to close
//TODO: Put this into the settings
bool active = false;          
//bool controlReleased = false;

//ports of controller/photo resistor
const uint8_t control = A4;
const uint8_t pr = A2;

//variables for the timer
unsigned long t0 = 0;
unsigned long t1 = 0;

//variables for saved values
unsigned long sumOfTheTimes = 0;
unsigned long lastTime = 0;
uint16_t numberMeasurements = 0;

//last state of the buttons
uint16_t lastButtons = 0;

void setup() {
  // put your setup code here, to run once:
  if(active){
    pinMode(control, OUTPUT);
  }
  else{
    pinMode(control, INPUT_PULLUP);
  }
  pinMode(pr, INPUT);  

  arcada.arcadaBegin();
  arcada.displayBegin();
  arcada.setBacklight(255); 
  
  Serial.begin(9600);
  
  drawBackground();
  
  if(setupMode){
    drawMenu();
  }
  else{
    drawLiveMonitor();
  }
}

void drawMenuEntry(uint8_t y, String text, uint16_t value, bool selected){
  //draw background and text for a single menu entry
  //TODO: This could be optimized

  //background
  arcada.display->fillRect(7, y-2, SC_XMAX-14, 30, ARCADA_DARKGREY);
    
  //basic border
  arcada.display->drawFastVLine(7, y-2, 31, ARCADA_DARKGREY);
  arcada.display->drawFastVLine(SC_XMAX-7, y-2, 30, ARCADA_LIGHTGREY);
  arcada.display->drawFastHLine(7, y-2, SC_XMAX-14, ARCADA_DARKGREY);
  arcada.display->drawFastHLine(8, y+28, SC_XMAX-14, ARCADA_LIGHTGREY);
    
  if(selected){
    arcada.display->fillRect(7, y-2, SC_XMAX-14, 30, ARCADA_DARKGREY);
      
    //arcada.display->drawFastVLine(7, y-2, 31, ARCADA_DARKGREY);
    arcada.display->drawFastVLine(SC_XMAX-7, y-2, 30, ARCADA_LIGHTGREY);
    //arcada.display->drawFastHLine(7, y-2, SC_XMAX-14, ARCADA_DARKGREY);
    arcada.display->drawFastHLine(8, y+28, SC_XMAX-14, ARCADA_LIGHTGREY);
      
    arcada.display->setTextColor(COLOR_ACCENT);
  }
  else{  
    arcada.display->fillRect(7, y-2, SC_XMAX-14, 30, ARCADA_LIGHTGREY);
      
    //arcada.display->drawFastVLine(7, y-2, 31, ARCADA_LIGHTGREY);
    arcada.display->drawFastVLine(SC_XMAX-7, y-2, 30, ARCADA_DARKGREY);
    //arcada.display->drawFastHLine(7, y-2, SC_XMAX-14, ARCADA_LIGHTGREY);
    arcada.display->drawFastHLine(8, y+28, SC_XMAX-14, ARCADA_DARKGREY);
      
    arcada.display->setTextColor(COLOR_TEXT);
  }
  
  arcada.display->setCursor(10, y);
  arcada.display->setTextSize(1);
  arcada.display->print(text); 
  
  arcada.display->setCursor(10, y + 10);
  arcada.display->setTextSize(2);
  arcada.display->print(value);
}

void drawMenu(){
  //draw the menu to make a basic setup 

  drawMenuEntry(17, "ambient light:", readRP(), false);
  drawMenuEntry(52, "threshold:", light_threshold, false);
  drawMenuEntry(87, "pulse width:", pulsewidth_ms, false);

  switch(selected){
    case 1:
      drawMenuEntry(17, "ambient light:", readRP(), true);
      break;
    case 2:
      drawMenuEntry(52, "threshold:", light_threshold, true);
      break;
    case 3:
      drawMenuEntry(87, "pulse width:", pulsewidth_ms, true);
      break;
  }
  
}

void drawLiveMonitor(){
  // draws the view for the live monitor, including text fields etc
  arcada.display->fillRect(7, 15, SC_XMAX-14, 30, ARCADA_LIGHTGREY);
  arcada.display->fillRect(7, 50, SC_XMAX-14, 30, ARCADA_LIGHTGREY);
  arcada.display->fillRect(7, 85, SC_XMAX-14, 30, ARCADA_LIGHTGREY);
  
  arcada.display->setTextColor(COLOR_TEXT);
  arcada.display->setTextSize(1);
  
  arcada.display->setCursor(10, 17);
  arcada.display->print("mean"); 
  arcada.display->setCursor(10, 52);
  arcada.display->print("total"); 
  arcada.display->setCursor(10, 87);
  arcada.display->print("last"); 
    
  arcada.display->setCursor(SC_XMAX-20, 31);
  arcada.display->print("ms"); 
  arcada.display->setCursor(SC_XMAX-20, 101);
  arcada.display->print("ms"); 

  displayTimes();  
}

void drawBackground(){
  // Draw the basic background for both setup and running modes
  
  //Draw background and set text settings
  arcada.display->fillScreen(COLOR_BG);
  arcada.display->fillRect(0, 0, SC_XMAX+1, 10, ARCADA_DARKGREY);
  
  arcada.display->setTextColor(COLOR_TEXT);
  arcada.display->setTextSize(1);

  // Draw task bar
  if(setupMode){
    arcada.display->setCursor(130, 1);
    arcada.display->print("start"); 
    drawMenu();
  }
  else{
    arcada.display->setCursor(1, 1);
    arcada.display->print("reset"); 
    arcada.display->setCursor(136, 1);
    arcada.display->print("menu"); 
    drawLiveMonitor();
  }
}

void displayTimes(){
  //calculate new values
  float mean = sumOfTheTimes/(numberMeasurements * 1000.0);
  float last = lastTime/1000.0;

  //values shouldn't overflow on the display bc it's ugly
  if(mean >= 1000000){
    mean = 999999.9;
  }
  else if(numberMeasurements == 0){
    mean = 0;
  }
  if(last >= 1000000){
    last = 999999.9;
  }

  //overwrite the old values
  arcada.display->fillRect(40, 15, SC_XMAX-60, 30, ARCADA_LIGHTGREY);
  arcada.display->fillRect(40, 50, SC_XMAX-60, 30, ARCADA_LIGHTGREY);
  arcada.display->fillRect(40, 85, SC_XMAX-60, 30, ARCADA_LIGHTGREY);
  
  //draw the new values
  arcada.display->setTextColor(COLOR_TEXT);
  arcada.display->setTextSize(2);
  arcada.display->setCursor(40, 24);
  arcada.display->print(mean, 1); 
  arcada.display->setCursor(40, 59);
  arcada.display->print(numberMeasurements); 
  arcada.display->setCursor(40, 94);
  
  if(last < 1000000){
    arcada.display->print(last, 1); 
  } 
  else{
    arcada.display->print("TIMEOUT");
  }
}

void updateTimes(long t){
  // adds a new measurement
  
  lastTime = t;
  sumOfTheTimes += lastTime;
  numberMeasurements++;  

  //display the time value
  displayTimes();
  
  Serial.println(t);
}

void generatePulse(){
  //generate an input to simulate the button being pressed
  digitalWrite(control, HIGH);
}

void pulseOff(){
  //turn the pulse off
  digitalWrite(control, LOW);
}

uint16_t readRP(){
  //read the input of the photo resistor
  return analogRead(pr);
  //return arcada.readLightSensor();
}

uint16_t readControl(){
  //this could be optimized by port reading
  return digitalRead(control);
}

void startMeasurement(){
  measurementRunning = true;
  //controlReleased = false;
  if(active){
    generatePulse();
  }
  t0 = micros();
}

void stopMeasurement(){
  measurementRunning = false;
  if(active){
    pulseOff();
  }
  t0 = micros();
}

void measurement(){
  // here we do measurements  
  if(measurementRunning){
    t1 = micros();
    if(readRP() > light_threshold){
      updateTimes(t1 - t0);
      stopMeasurement();
    }
    else if(t1 - t0 > TIMEOUT){
      lastTime = 1000000;
      displayTimes();
      stopMeasurement();
      Serial.println("TIMEOUT");
    }
    else if(active && t1 - t0 > pulsewidth_ms * 1000){
      pulseOff();
    }
  }
  else {
    if(micros() - t0 > downtime_ms * 1000){
      //if(readControl() && !active){
      //  controlReleased = true;
      //}
      if(active){
        startMeasurement();   
      }
      else if(!readControl()){ // && controlReleased){
        startMeasurement();
      }
    }
  }
}

void increaseValue(){
  //increases the value of the selected menu entry
  switch(selected){
    case 1: 
      break;
    case 2:
      light_threshold += 10;
      if(light_threshold > 1000){
        light_threshold = 1000;
      }
      break;
    case 3:
      if(pulsewidth_ms < 100){
        pulsewidth_ms++;
      }
      break;
  }
}

void decreaseValue(){
  //decreases the value of the selected menu entry
  switch(selected){
    case 1: 
      break;
    case 2:
      light_threshold -= 10;
      if(light_threshold < 0){
        light_threshold = 0;
      }
      break;
    case 3:
      if(pulsewidth_ms > 1){
        pulsewidth_ms--;
      }
      break;
  }
}

uint16_t buttonsNew, buttons;

void loop() {
  buttonsNew = arcada.readButtons();
  buttons = buttonsNew & ~lastButtons;

  if(setupMode){
    if(buttons & ARCADA_BUTTONMASK_UP && selected > 1){
      selected--;
      drawMenu();
    }
    else if(buttons & ARCADA_BUTTONMASK_DOWN && selected < 3){
      selected++;
      drawMenu();
    }

    if(buttons & ARCADA_BUTTONMASK_A){
      increaseValue();
      drawMenu();
    }
    else if(buttons & ARCADA_BUTTONMASK_B){
      decreaseValue();
      drawMenu();
    }
    
    if(buttons & ARCADA_BUTTONMASK_START){
      setupMode = false;
      drawBackground();
    }

    //this still flickers awfully, but it refreshes the ambient light indicator
    /*if(micros() - t0 > 200000){
      drawMenu();
      t0 = micros();
    }*/
  }
  else{
    measurement();
    
    if(buttons & ARCADA_BUTTONMASK_START){
      setupMode = true;
      drawBackground();
    }
    else if(buttons & ARCADA_BUTTONMASK_SELECT){
      //reset the saved times
      sumOfTheTimes = 0;
      lastTime = 0;
      numberMeasurements = 0;
      displayTimes();
      stopMeasurement();
    }
  }
  
  lastButtons = buttonsNew;
}
