use ncurses::*;
use gilrs::{Gilrs, Event};
use gilrs::EventType::{ButtonPressed, AxisChanged};

use std::env;

use std::fs::File;
use std::io::prelude::*;
use std::{thread,time};

fn main() -> std::io::Result<()> {
    let mut green = File::create("/sys/class/leds/led0/brightness")?;
    let mut red = File::create("/sys/class/leds/led1/brightness")?;
    // initialiye by turning both leds off
    green.write(b"0")?;
    red.write(b"0")?;
    initscr();
    start_color();
    use_default_colors();
    noecho();
    scrollok(stdscr(),true);

    // bg white
    init_pair(1, COLOR_BLACK, COLOR_WHITE);
    // bg black
    init_pair(2, COLOR_WHITE, COLOR_BLACK);
    // init screen as black
    wbkgd(stdscr(), COLOR_PAIR(2));

    if is_gamepad_mode() {
        // Gamepad setup
        let mut gilrs = Gilrs::new().unwrap();

        // Iterate over all connected gamepads
        for (_id, gamepad) in gilrs.gamepads() {
            addstr(&format!("{} is {:?}\n", gamepad.name(), gamepad.power_info()));
            refresh();
        }

        loop {
            let mut output_string = String::new();
            let now = time::Instant::now();
            
            match gilrs.next_event() {
                Some(Event {event, ..}) => {
                    if let ButtonPressed(_, _) = event {
                        red.write(b"1")?;
                        output_string = format!("{:?}\n", now.elapsed());
                    }

                    if let AxisChanged(_, amount, ..) = event {
                        if amount != 0.0 {
                            red.write(b"1")?;
                            output_string = format!("{:?}", now.elapsed());
                            output_string += &format!("\tAxis Change: {}\n", amount);

                        }
                    }
                    
                    // if the output string is still empty, then another action than the necessary 2
                    // was done. skip
                    if !output_string.is_empty() {
                        addstr(&output_string);
                        wbkgd(stdscr(), COLOR_PAIR(1));
                        refresh();
                        thread::sleep(time::Duration::from_millis(50));
                        red.write(b"0")?;
                        wbkgd(stdscr(), COLOR_PAIR(2));
                        refresh();
                    }
                }
                _ => ()
            }
        }
    }
    // we are in keyboard mode
    else {
        loop {
            getch();
            let now = time::Instant::now();
            red.write(b"1")?;
            addstr(&format!("{:?}\n", now.elapsed()));
            wbkgd(stdscr(), COLOR_PAIR(1));
            refresh();
            thread::sleep(time::Duration::from_millis(50));
            red.write(b"0")?;
            wbkgd(stdscr(), COLOR_PAIR(2));
            refresh();
        }
    }
}

fn is_gamepad_mode() -> bool {
    let args: Vec<String> = env::args().collect();

    args.iter().any(|i| i=="g")
}
